﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;

namespace _20190218_TicketViewAnalyseTerre
{
    public class CoucheFactory
    {
        private List<Layer> layers;
        private double zoomMin;
        private double ZoomMax;

        public List<Layer> Layers
        {
            get { return layers; }
            private set { layers = value; }
        }

        public RectangleShape BoundingBox { get; private set; }

        public CoucheFactory(XmlCoucheCollection couches,List<ConstantsMap.TypeCouche> select)
        {
            layers = new List<Layer>();

            if (couches.XmlCouches == null)
                return;

            foreach (var c in couches.XmlCouches)
            {
                if (select.Contains(c.Type))
                {
                    switch (c.Type)
                    {
                        case ConstantsMap.TypeCouche.shp:
                            layers.Add(initialiserCoucheShp(c));                 
                        break;

                        case ConstantsMap.TypeCouche.raster:                
                            layers.Add(initialiserCoucheRaster(c));
                   
                        break;

                        case ConstantsMap.TypeCouche.sql:
                            layers.Add(initialiserCoucheSql(c));
                   
                        break;
                    }
                }
            }
        }

        private ShapeFileFeatureLayer initialiserCoucheShp(XmlCouche coucheXml)
        {
            var rep = OutilsOvLyr.Path("SHP\\")+coucheXml.Repertoire;
            ShapeFileFeatureLayer sh = new ShapeFileFeatureLayer(rep);
            sh.Name = coucheXml.Nom;
            ShapeFileFeatureSource.BuildIndexFile(rep, BuildIndexMode.DoNotRebuild);
            sh.RequireIndex = true;

            sh.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, ConstantsMap.RenvoyerCouleur(coucheXml.Couleur), 3));
            sh.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            sh.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(TextStyles.CreateSimpleTextStyle(coucheXml.Libelle, "Arial", 10, DrawingFontStyles.Bold, ConstantsMap.RenvoyerCouleur(coucheXml.Couleur)));
            sh.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            if (coucheXml.CoucheBase)
            {
                zoomMin = sh.ZoomLevelSet.ZoomLevel01.Scale;
                ZoomMax = sh.ZoomLevelSet.ZoomLevel20.Scale;

                try
                {
                    sh.Open();
                    BoundingBox = sh.GetBoundingBox();
                }
                catch (Exception ex)
                {

                }
            }

            return sh;
        }

        private MultiGeoRasterLayer initialiserCoucheRaster(XmlCouche coucheXml)
        {
            var repertoireTxt = Path.Combine(OutilsOvLyr.PathRefTxtRaster, ConstantsMap.REFERENCETXT);
            MultiGeoRasterLayer multiRaster = new MultiGeoRasterLayer(repertoireTxt, 590591790.0, 1126.4644432067871, coucheXml.Nom);
            multiRaster.BuildReferenceFile(repertoireTxt, OutilsOvLyr.Path(coucheXml.Repertoire));
            return multiRaster;
        }

        private MsSqlFeatureLayer initialiserCoucheSql(XmlCouche coucheXml)
        {
            var layer = new MsSqlFeatureLayer(coucheXml.Repertoire,coucheXml.SchemaTable.Split('.').Last(),coucheXml.FeatureIdColumn);
            layer.CustomGeometryColumnName = coucheXml.CustomGeometryColumnName;
            layer.SchemaName = coucheXml.SchemaTable.Split('.').First();
            layer.Name = coucheXml.Libelle;
            layer.Srid = 2154;
            if (coucheXml.Style?.ToLower() != "point")
            {
                layer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(AreaStyles.CreateSimpleAreaStyle(
                    GeoColor.SimpleColors.Transparent, ConstantsMap.RenvoyerCouleur(coucheXml.Couleur), 3));
            }
            else
            {
                layer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(PointStyles.CreateSimpleCircleStyle(
                     ConstantsMap.RenvoyerCouleur(coucheXml.Couleur), 20,GeoColors.Black,3));

                layer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(AreaStyles.CreateSimpleAreaStyle(
                    GeoColor.SimpleColors.Transparent, ConstantsMap.RenvoyerCouleur(coucheXml.Couleur), 20));
            }

            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            layer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(TextStyles.CreateSimpleTextStyle(coucheXml.Libelle, "Arial", 10, DrawingFontStyles.Bold, GeoColor.SimpleColors.Black));
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            return layer;
        }
    }
}
